@@ -31,13 +31,12 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <gconf/gconf-client.h>
 
 #define SN_API_NOT_YET_FROZEN
 #include <libsn/sn.h>
 #include <gdk/gdk.h>
-#include <gdk/gdkx.h>
+#include <gdk/gdkkeysyms.h>
 
 #include <totem-pl-parser.h>
 
@@ -49,7 +48,9 @@
 #include "bacon-video-widget.h"
 #include "totem-interface.h"
 #include "totem-statusbar.h"
-#include "bacon-volume.h"
+#include "totem-time-label.h"
+#include "totem-fullscreen.h"
+#include "totem-glow-button.h"
 #include "video-utils.h"
 
 #include "totem-plugin-viewer-commands.h"
@@ -58,9 +59,10 @@
 
 GtkWidget *totem_statusbar_create (void);
 GtkWidget *totem_volume_create (void);
+GtkWidget *totem_pp_create (void);
 
-#define VOLUME_DOWN_OFFSET -8
-#define VOLUME_UP_OFFSET 8
+#define VOLUME_DOWN_OFFSET (-0.08)
+#define VOLUME_UP_OFFSET (0.08)
 
 /* For newer D-Bus version */
 #ifndef DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT
@@ -92,12 +94,20 @@ typedef enum {
 
 typedef GObjectClass TotemEmbeddedClass;
 
+typedef struct _TotemPlItem {
+	char *uri;
+	int duration;
+	int starttime;
+} TotemPlItem;
+
 typedef struct _TotemEmbedded {
 	GObject parent;
 
 	GtkWidget *window;
-	GladeXML *menuxml, *xml;
+	GtkBuilder *menuxml, *xml;
 	GtkWidget *about;
+	GtkWidget *pp_button;
+	GtkWidget *pp_fs_button;
 	TotemStatusbar *statusbar;
 	int width, height;
 	const char *mimetype;
@@ -110,7 +120,7 @@ typedef struct _TotemEmbedded {
 	TotemStates state;
 	GdkCursor *cursor;
 
-	/* Playlist */
+	/* Playlist, a GList of TotemPlItem */
 	GList *playlist, *current;
 	guint parser_id;
 	int num_items;
@@ -123,6 +133,13 @@ typedef struct _TotemEmbedded {
 	GtkAdjustment *seekadj;
 	GtkWidget *seek;
 
+	/* Volume */
+	GtkWidget *volume;
+
+	/* Fullscreen */
+	TotemFullscreen *fs;
+	GtkWidget * fs_window;
+
 	/* Error */
 
 	guint type : 3; /* TotemPluginType */
@@ -134,7 +151,6 @@ typedef struct _TotemEmbedded {
 	guint hidden : 1;
 	guint repeat : 1;
 	guint seeking : 1;
-	guint noautostart : 1;
 	guint autostart : 1;
 	guint audioonly : 1;
 } TotemEmbedded;
@@ -165,6 +181,12 @@ static void totem_embedded_clear_playlist (TotemEmbedded *embedded);
 
 static void totem_embedded_update_menu (TotemEmbedded *emb);
 static void on_open1_activate (GtkButton *button, TotemEmbedded *emb);
+static void totem_embedded_toggle_fullscreen (TotemEmbedded *emb);
+
+void on_about1_activate (GtkButton *button, TotemEmbedded *emb);
+void on_preferences1_activate (GtkButton *button, TotemEmbedded *emb);
+void on_copy_location1_activate (GtkButton *button, TotemEmbedded *emb);
+void on_fullscreen1_activate (GtkMenuItem *menuitem, TotemEmbedded *emb);
 
 enum {
 	BUTTON_PRESS,
@@ -186,6 +208,8 @@ totem_embedded_finalize (GObject *object)
 		g_object_unref (embedded->xml);
 	if (embedded->menuxml)
 		g_object_unref (embedded->menuxml);
+	if (embedded->fs)
+		g_object_unref (embedded->fs);
 
 	/* FIXME etc */
 
@@ -244,6 +268,16 @@ totem_embedded_error_and_exit (char *title, char *reason, TotemEmbedded *emb)
 	totem_embedded_exit (emb);
 }
 
+static void
+totem_embedded_save_volume (TotemEmbedded *emb, double volume)
+{
+	GConfClient *gc;
+
+	gc = gconf_client_get_default ();
+	gconf_client_set_int (gc, GCONF_PREFIX"/volume", (int) (volume * 100.0), NULL);
+	g_object_unref (G_OBJECT (gc));
+}
+
 static void
 totem_embedded_set_error (TotemEmbedded *emb,
 			  char *primary,
@@ -257,7 +291,7 @@ static void
 totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 {
 	GtkWidget *image;
-	char id[32];
+	gchar *id;
 	static const char *states[] = {
 		"PLAYING",
 		"PAUSED",
@@ -270,14 +304,15 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 
 	g_message ("Viewer state: %s", states[state]);
 
-	image = glade_xml_get_widget (emb->xml, "emb_pp_button_image");
+	image = gtk_button_get_image (GTK_BUTTON (emb->pp_button));
 
 	switch (state) {
 	case STATE_STOPPED:
-		g_snprintf (id, sizeof (id), "gtk-media-play-%s",
-			    gtk_widget_get_direction (image) ? "ltr" : "rtl");
+		id = GTK_STOCK_MEDIA_PLAY;
 		totem_statusbar_set_text (emb->statusbar, _("Stopped"));
 		totem_statusbar_set_time_and_length (emb->statusbar, 0, 0);
+		totem_time_label_set_time 
+			(TOTEM_TIME_LABEL (emb->fs->time_label), 0, 0);
 		if (emb->href_uri != NULL && emb->hidden == FALSE) {
 			gdk_window_set_cursor
 				(GTK_WIDGET (emb->bvw)->window,
@@ -285,12 +320,11 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 		}
 		break;
 	case STATE_PAUSED:
-		g_snprintf (id, sizeof (id), "gtk-media-play-%s",
-				gtk_widget_get_direction (image) ? "ltr" : "rtl");
+		id = GTK_STOCK_MEDIA_PLAY;
 		totem_statusbar_set_text (emb->statusbar, _("Paused"));
 		break;
 	case STATE_PLAYING:
-		g_snprintf (id, sizeof (id), "gtk-media-pause");
+		id = GTK_STOCK_MEDIA_PAUSE;
 		totem_statusbar_set_text (emb->statusbar, _("Playing"));
 		if (emb->href_uri == NULL && emb->hidden == FALSE) {
 			gdk_window_set_cursor
@@ -299,10 +333,13 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 		}
 		break;
 	default:
+		g_assert_not_reached ();
 		break;
 	}
 
-	gtk_image_set_from_icon_name (GTK_IMAGE (image), id, GTK_ICON_SIZE_MENU);
+	gtk_image_set_from_stock (GTK_IMAGE (image), id, GTK_ICON_SIZE_MENU);
+	gtk_tool_button_set_stock_id (GTK_TOOL_BUTTON (emb->pp_fs_button), id);
+
 	emb->state = state;
 }
 
@@ -313,6 +350,7 @@ totem_embedded_pad_pixbuf_for_size (GdkPixbuf *pixbuf,
 	GdkPixbuf *logo;
 	guchar *pixels;
 	int rowstride, i;
+	int dest_x, dest_y;
 
 	logo = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
 			       TRUE, 8, width, height);
@@ -324,13 +362,15 @@ totem_embedded_pad_pixbuf_for_size (GdkPixbuf *pixbuf,
 		memset (pixels + i * rowstride, 0, width * 4);
 	}
 
+	dest_x = (width - gdk_pixbuf_get_width (pixbuf)) / 2;
+	dest_y = (height - gdk_pixbuf_get_height (pixbuf)) / 2;
 	gdk_pixbuf_copy_area (pixbuf,
 			      0, 0,
-			      gdk_pixbuf_get_width (pixbuf),
-			      gdk_pixbuf_get_height (pixbuf),
+			      MIN (gdk_pixbuf_get_width (logo), gdk_pixbuf_get_width (pixbuf)),
+			      MIN (gdk_pixbuf_get_height (logo), gdk_pixbuf_get_height (pixbuf)),
 			      logo,
-			      (width - gdk_pixbuf_get_width (pixbuf)) / 2,
-			      (height - gdk_pixbuf_get_height (pixbuf)) / 2);
+			      dest_x,
+			      dest_y);
 
 	return logo;
 }
@@ -370,14 +410,13 @@ totem_embedded_set_logo_by_name (TotemEmbedded *embedded,
 static void
 totem_embedded_set_pp_state (TotemEmbedded *emb, gboolean state)
 {
-	GtkWidget *item;
-
-	item = glade_xml_get_widget (emb->xml, "pp_button");
-	gtk_widget_set_sensitive (item, state);
+	gtk_widget_set_sensitive (emb->pp_button, state);
+	gtk_widget_set_sensitive (emb->pp_fs_button, state);
 }
 
 static gboolean
 totem_embedded_open_internal (TotemEmbedded *emb,
+			      gboolean start_play,
 			      GError **error)
 {
 	GError *err = NULL;
@@ -401,7 +440,7 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 		return FALSE;
 	}
 
-	g_message ("totem_embedded_open_internal '%s' is-browser-stream %d", uri, emb->is_browser_stream);
+	g_message ("totem_embedded_open_internal '%s' is-browser-stream %d start-play %d", uri, emb->is_browser_stream, start_play);
 
 	bacon_video_widget_set_logo_mode (emb->bvw, FALSE);
 
@@ -434,13 +473,18 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 
 		totem_embedded_set_pp_state (emb, FALSE);
 	} else {
-		if (emb->playlist == NULL && totem_embedded_play (emb, NULL) != FALSE) {
-			totem_embedded_set_state (emb, STATE_PLAYING);
-		}
-		totem_embedded_set_pp_state (emb, TRUE);
+		/* FIXME we shouldn't even do that here */
+		if (start_play)
+			totem_embedded_play (emb, NULL);
+		else
+			totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
 	}
 
 	totem_embedded_update_menu (emb);
+	if (emb->href_uri != NULL)
+		totem_fullscreen_set_title (emb->fs, emb->href_uri);
+	else
+		totem_fullscreen_set_title (emb->fs, emb->current_uri);
 
 	return retval;
 }
@@ -453,6 +497,7 @@ totem_embedded_play (TotemEmbedded *emb,
 
 	if (bacon_video_widget_play (emb->bvw, &err) != FALSE) {
 		totem_embedded_set_state (emb, STATE_PLAYING);
+		totem_embedded_set_pp_state (emb, TRUE);
 	} else {
 		g_warning ("Error in bacon_video_widget_play: %s", err->message);
 		g_error_free (err);
@@ -871,6 +916,8 @@ totem_embedded_set_uri (TotemEmbedded *emb,
 	g_free (old_uri);
 	g_free (old_base);
 	g_free (old_href);
+	g_free (emb->stream_uri);
+	emb->stream_uri = NULL;
 }
 
 static gboolean
@@ -885,7 +932,7 @@ totem_embedded_open_uri (TotemEmbedded *emb,
 
 	totem_embedded_set_uri (emb, uri, base_uri, FALSE);
 
-	return totem_embedded_open_internal (emb, error);
+	return totem_embedded_open_internal (emb, TRUE, error);
 }
 
 static gboolean
@@ -902,7 +949,7 @@ totem_embedded_open_stream (TotemEmbedded *emb,
 
 	/* FIXME: consume any remaining input from stdin */
 
-	return totem_embedded_open_internal (emb, error);
+	return totem_embedded_open_internal (emb, TRUE, error);
 }
 
 static gboolean
@@ -922,6 +969,7 @@ static gboolean
 totem_embedded_open_playlist_item (TotemEmbedded *emb,
 				   GList *item)
 {
+	TotemPlItem *plitem;
 	gboolean eop;
 
 	if (!emb->playlist)
@@ -937,16 +985,29 @@ totem_embedded_open_playlist_item (TotemEmbedded *emb,
 	emb->current = item;
 	g_assert (item != NULL);
 
+	plitem = (TotemPlItem *) item->data;
+
 	totem_embedded_set_uri (emb,
-				(const char *) item->data,
+				(const char *) plitem->uri,
 			        emb->base_uri /* FIXME? */,
 			        FALSE);
 
 	bacon_video_widget_close (emb->bvw);
-	if (totem_embedded_open_internal (emb, NULL /* FIXME */) &&
-	    ((eop != FALSE && emb->repeat != FALSE)
-		|| (eop == FALSE))) {
-		totem_embedded_play (emb, NULL);
+	if (totem_embedded_open_internal (emb, FALSE, NULL /* FIXME */)) {
+		if (plitem->starttime > 0) {
+			gboolean retval;
+
+			g_message ("Seeking to %d seconds for starttime", plitem->starttime);
+			retval = bacon_video_widget_seek_time (emb->bvw,
+							       plitem->starttime * 1000,
+							       NULL /* FIXME */);
+			if (!retval)
+				return TRUE;
+		}
+
+		if ((eop != FALSE && emb->repeat != FALSE) || (eop == FALSE)) {
+		    	    totem_embedded_play (emb, NULL);
+		}
 	}
 
 	return TRUE;
@@ -973,7 +1034,28 @@ totem_embedded_set_local_file (TotemEmbedded *emb,
 	totem_embedded_set_uri (emb, file_uri, base_uri, FALSE);
 	g_free (file_uri);
 
-	return totem_embedded_open_internal (emb, error);
+	return totem_embedded_open_internal (emb, TRUE, error);
+}
+
+static gboolean
+totem_embedded_set_local_cache (TotemEmbedded *emb,
+				const char *path,
+				GError **error)
+{
+	char *file_uri;
+
+	/* FIXME Should also handle playlists */
+	if (!emb->is_browser_stream)
+		return TRUE;
+
+	file_uri = g_filename_to_uri (path, NULL, error);
+	if (!file_uri)
+		return FALSE;
+
+	emb->stream_uri = emb->current_uri;
+	emb->current_uri = file_uri;
+
+	return TRUE;
 }
 
 static gboolean
@@ -1008,7 +1090,8 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 static void
 totem_embedded_update_menu (TotemEmbedded *emb)
 {
-	GtkWidget *menu, *item, *image;
+	GtkWidget *item, *image;
+	GtkMenuShell *menu;
 	char *label;
 
 	if (emb->menu_item != NULL) {
@@ -1044,7 +1127,7 @@ totem_embedded_update_menu (TotemEmbedded *emb)
 	/* translators: this is:
 	 * Open With ApplicationName
 	 * as in nautilus' right-click menu */
-	label = g_strdup_printf ("_Open with \"%s\"", emb->app->name);
+	label = g_strdup_printf (_("_Open with \"%s\""), emb->app->name);
 	item = gtk_image_menu_item_new_with_mnemonic (label);
 	g_free (label);
 	image = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
@@ -1054,8 +1137,8 @@ totem_embedded_update_menu (TotemEmbedded *emb)
 	gtk_widget_show (item);
 	emb->menu_item = item;
 
-	menu = glade_xml_get_widget (emb->menuxml, "menu");
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	menu = GTK_MENU_SHELL (gtk_builder_get_object (emb->menuxml, "menu"));
+	gtk_menu_shell_prepend (menu, item);
 }
 
 static void
@@ -1067,7 +1150,14 @@ on_open1_activate (GtkButton *button, TotemEmbedded *emb)
 	totem_embedded_stop (emb, NULL);
 }
 
-static void
+void
+on_fullscreen1_activate (GtkMenuItem *menuitem, TotemEmbedded *emb)
+{
+	if (totem_fullscreen_is_fullscreen (emb->fs) == FALSE)
+		totem_embedded_toggle_fullscreen (emb);
+}
+
+void
 on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 {
 	char *backend_version, *description, *license;
@@ -1121,7 +1211,7 @@ on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 	gtk_widget_show (emb->about);
 }
 
-static void
+void
 on_copy_location1_activate (GtkButton *button, TotemEmbedded *emb)
 {
 	GdkDisplay *display;
@@ -1146,12 +1236,20 @@ on_copy_location1_activate (GtkButton *button, TotemEmbedded *emb)
 	gtk_clipboard_set_text (clip, uri, -1);
 }
 
+void
+on_preferences1_activate (GtkButton *button, TotemEmbedded *emb)
+{
+	/* TODO: */
+}
+
 static void
 on_play_pause (GtkWidget *widget, TotemEmbedded *emb)
 {
 	if (emb->state == STATE_PLAYING) {
 		totem_embedded_pause (emb, NULL);
 	} else {
+		totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), FALSE);
+
 		if (emb->current_uri == NULL) {
 			g_signal_emit (emb, signals[BUTTON_PRESS], 0,
 				       GDK_CURRENT_TIME, 0);
@@ -1171,7 +1269,11 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 
 	bacon_video_widget_close (emb->bvw);
 
-	if (emb->current_uri)
+	/* We are using a local cache, so we resolve against the stream_uri */
+	if (emb->stream_uri)
+		new_uri = totem_resolve_relative_link (emb->stream_uri, mrl);
+	/* We don't have a local cache, so resolve against the URI */
+	else if (emb->current_uri)
 		new_uri = totem_resolve_relative_link (emb->current_uri, mrl);
 	/* FIXME: not sure that this is actually correct... */
 	else if (emb->base_uri)
@@ -1189,8 +1291,42 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 
 	totem_embedded_set_state (emb, STATE_STOPPED);
 
-	if (totem_embedded_open_internal (emb, NULL /* FIXME? */) != FALSE)
-		totem_embedded_play (emb, NULL);
+	totem_embedded_open_internal (emb, TRUE, NULL /* FIXME? */);
+}
+
+static void
+totem_embedded_toggle_fullscreen (TotemEmbedded *emb)
+{
+	GtkAction * fs_action = GTK_ACTION (gtk_builder_get_object 
+					    (emb->menuxml, "fullscreen1"));
+
+	if (totem_fullscreen_is_fullscreen (emb->fs) != FALSE)
+	{
+		GtkWidget * container;
+		container = GTK_WIDGET (gtk_builder_get_object (emb->xml,
+								"video_box"));
+
+		totem_fullscreen_set_fullscreen (emb->fs, FALSE);
+		gtk_widget_reparent (GTK_WIDGET (emb->bvw), container);
+		gtk_widget_hide_all (emb->fs_window);
+		
+		gtk_action_set_sensitive (fs_action, TRUE);
+	} else {
+		gtk_widget_reparent (GTK_WIDGET (emb->bvw), emb->fs_window);
+		bacon_video_widget_set_fullscreen (emb->bvw, TRUE);
+		gtk_window_fullscreen (GTK_WINDOW (emb->fs_window));
+		totem_fullscreen_set_fullscreen (emb->fs, TRUE);
+		gtk_widget_show_all (emb->fs_window);
+
+		gtk_action_set_sensitive (fs_action, FALSE);
+	}
+}
+
+static void
+totem_embedded_on_fullscreen_exit (GtkWidget *widget, TotemEmbedded *emb)
+{
+	if (totem_fullscreen_is_fullscreen (emb->fs) != FALSE)
+		totem_embedded_toggle_fullscreen (emb);
 }
 
 static gboolean
@@ -1217,7 +1353,7 @@ on_video_button_press_event (BaconVideoWidget *bvw,
 		   state == 0) {
 		GtkMenu *menu;
 
-		menu = GTK_MENU (glade_xml_get_widget (emb->menuxml, "menu"));
+		menu = GTK_MENU (gtk_builder_get_object (emb->menuxml, "menu"));
 		gtk_menu_popup (menu, NULL, NULL, NULL, NULL,
 				event->button, event->time);
 
@@ -1228,18 +1364,29 @@ on_video_button_press_event (BaconVideoWidget *bvw,
 }
 
 static void
-on_eos_event (GtkWidget *bvw, TotemEmbedded *emb)
+on_eos_event (BaconVideoWidget *bvw, TotemEmbedded *emb)
 {
+	gboolean start_play;
+
 	totem_embedded_set_state (emb, STATE_STOPPED);
 	gtk_adjustment_set_value (emb->seekadj, 0);
 
 	/* FIXME: the plugin needs to handle EOS itself, e.g. for QTNext */
 
+	start_play = (emb->repeat != FALSE && emb->autostart);
+
 	/* No playlist if we have fd://0, right? */
 	if (emb->is_browser_stream) {
-		/* FIXME: totem_embedded_set_state missing? */
-		/* FIXME: should find a way to enable playback of the stream again without re-requesting it */
-		totem_embedded_set_pp_state (emb, FALSE);
+		/* Verify that we had a SetLocalCache */
+		if (g_str_has_prefix (emb->current_uri, "file://") != FALSE) {
+			emb->num_items = 1;
+			emb->is_browser_stream = FALSE;
+			bacon_video_widget_close (emb->bvw);
+			totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
+		} else {
+			/* FIXME: should find a way to enable playback of the stream again without re-requesting it */
+			totem_embedded_set_pp_state (emb, FALSE);
+		}
 	/* FIXME? else if (emb->playing_nth_item == emb->playlist_num_items) ? */
 	} else if (emb->num_items == 1) {
 		if (g_str_has_prefix (emb->current_uri, "file://") != FALSE) {
@@ -1248,19 +1395,24 @@ on_eos_event (GtkWidget *bvw, TotemEmbedded *emb)
 				bacon_video_widget_seek (emb->bvw, 0.0, NULL);
 			} else {
 				bacon_video_widget_close (emb->bvw);
-				totem_embedded_open_internal (emb, NULL /* FIXME? */);
+				totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
 			}
 		} else {
 			bacon_video_widget_close (emb->bvw);
-			totem_embedded_open_internal (emb, NULL /* FIXME? */);
+			totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
 		}
-		if (emb->repeat != FALSE && emb->autostart)
-			totem_embedded_play (emb, NULL);
 	} else if (emb->current) {
 		totem_embedded_open_playlist_item (emb, emb->current->next);
 	}
 }
 
+static gboolean
+skip_unplayable_stream (TotemEmbedded *emb)
+{
+	on_eos_event (BACON_VIDEO_WIDGET (emb->bvw), emb);
+	return FALSE;
+}
+
 static void
 on_error_event (BaconVideoWidget *bvw,
 		char *message,
@@ -1274,6 +1426,24 @@ on_error_event (BaconVideoWidget *bvw,
 			g_signal_emit (emb, signals[STOP_STREAM], 0);
 	
 		totem_embedded_set_state (emb, STATE_STOPPED);
+
+		/* If we have a playlist, and that the current item
+		 * is < 60 seconds long, just go through it
+		 *
+		 * Same thing for all the items in a non-repeat playlist,
+		 * other than the last one
+		 *
+		 * FIXME we should mark streams as not playable though
+		 * so we don't loop through unplayable streams... */
+		if (emb->num_items > 1 && emb->current) {
+			TotemPlItem *item = emb->current->data;
+
+			if ((item->duration > 0 && item->duration < 60)
+			    || (!emb->repeat && emb->current->next)) {
+				g_idle_add ((GSourceFunc) skip_unplayable_stream, emb);
+				return;
+			}
+		}
 	}
 
 	if (fatal) {
@@ -1285,35 +1455,9 @@ on_error_event (BaconVideoWidget *bvw,
 }
 
 static void
-cb_vol (GtkWidget *val, TotemEmbedded *emb)
+cb_vol (GtkWidget *val, gdouble value, TotemEmbedded *emb)
 {
-	bacon_video_widget_set_volume (emb->bvw,
-		bacon_volume_button_get_value (BACON_VOLUME_BUTTON (val)));
-}
-
-static gboolean
-on_volume_scroll_event (GtkWidget *win, GdkEventScroll *event, TotemEmbedded *emb)
-{
-	GtkWidget *vbut;
-	int vol, offset;
-
-	switch (event->direction) {
-	case GDK_SCROLL_UP:
-		offset = VOLUME_UP_OFFSET;
-		break;
-	case GDK_SCROLL_DOWN:
-		offset = VOLUME_DOWN_OFFSET;
-		break;
-	default:
-		return FALSE;
-	}
-
-	vbut = glade_xml_get_widget (emb->xml, "volume_button");
-	vol = bacon_volume_button_get_value (BACON_VOLUME_BUTTON (vbut));
-	bacon_volume_button_set_value (BACON_VOLUME_BUTTON (vbut),
-			vol + offset);
-
-	return FALSE;
+	bacon_video_widget_set_volume (emb->bvw, value);
 }
 
 static void
@@ -1326,6 +1470,7 @@ on_tick (GtkWidget *bvw,
 {
 	if (emb->state != STATE_STOPPED) {
 		gtk_widget_set_sensitive (emb->seek, seekable);
+		gtk_widget_set_sensitive (emb->fs->seek, seekable);
 		if (emb->seeking == FALSE)
 			gtk_adjustment_set_value (emb->seekadj,
 					current_position * 65535);
@@ -1337,13 +1482,39 @@ on_tick (GtkWidget *bvw,
 					(int) (current_time / 1000),
 					(int) (stream_length / 1000));
 		}
+
+		totem_time_label_set_time 
+			(TOTEM_TIME_LABEL (emb->fs->time_label),
+			 current_time, stream_length);
 	}
 }
 
+static void
+on_buffering (BaconVideoWidget *bvw, guint percentage, TotemEmbedded *emb)
+{
+//	g_message ("Buffering: %d %%", percentage);
+}
+
+static void
+property_notify_cb_volume (BaconVideoWidget *bvw, GParamSpec *spec,
+			   TotemEmbedded *emb)
+{
+	double volume;
+	
+	volume = bacon_video_widget_get_volume (emb->bvw);
+	
+	g_signal_handlers_block_by_func (emb->volume, cb_vol, emb);
+	gtk_scale_button_set_value (GTK_SCALE_BUTTON (emb->volume), volume);
+	totem_embedded_save_volume (emb, volume);
+	g_signal_handlers_unblock_by_func (emb->volume, cb_vol, emb);
+}
+
 static gboolean
 on_seek_start (GtkWidget *widget, GdkEventButton *event, TotemEmbedded *emb)
 {
 	emb->seeking = TRUE;
+	totem_time_label_set_seeking (TOTEM_TIME_LABEL (emb->fs->time_label),
+				      TRUE);
 
 	return FALSE;
 }
@@ -1353,6 +1524,8 @@ cb_on_seek (GtkWidget *widget, GdkEventButton *event, TotemEmbedded *emb)
 {
 	bacon_video_widget_seek (emb->bvw,
 		gtk_range_get_value (GTK_RANGE (widget)) / 65535, NULL);
+	totem_time_label_set_seeking (TOTEM_TIME_LABEL (emb->fs->time_label),
+				      FALSE);
 	emb->seeking = FALSE;
 
 	return FALSE;
@@ -1369,17 +1542,91 @@ controls_size_allocate_cb (GtkWidget *controls,
 }
 #endif
 
+static void
+totem_embedded_action_volume_relative (TotemEmbedded *emb, double off_pct)
+{
+	double vol;
+	
+	if (bacon_video_widget_can_set_volume (emb->bvw) == FALSE)
+		return;
+	
+	vol = bacon_video_widget_get_volume (emb->bvw);
+	bacon_video_widget_set_volume (emb->bvw, vol + off_pct);
+}
+
+static gboolean
+totem_embedded_handle_key_press (TotemEmbedded *emb, GdkEventKey *event)
+{
+	switch (event->keyval) {
+	case GDK_Escape:
+		if (totem_fullscreen_is_fullscreen (emb->fs) != FALSE)
+			totem_embedded_toggle_fullscreen (emb);
+		return TRUE;
+	case GDK_F11:
+	case GDK_f:
+	case GDK_F:
+		totem_embedded_toggle_fullscreen (emb);
+		return TRUE;
+	case GDK_space:
+		on_play_pause (NULL, emb);
+		return TRUE;
+	case GDK_Up:
+		totem_embedded_action_volume_relative (emb, VOLUME_UP_OFFSET);
+		return TRUE;
+	case GDK_Down:
+		totem_embedded_action_volume_relative (emb, VOLUME_DOWN_OFFSET);
+		return TRUE;
+	case GDK_Left:
+	case GDK_Right:
+		/* FIXME: */
+		break;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+totem_embedded_key_press_event (GtkWidget *widget, GdkEventKey *event,
+				TotemEmbedded *emb)
+{
+	if (event->type != GDK_KEY_PRESS)
+		return FALSE;
+	
+	if (event->state & GDK_CONTROL_MASK)
+	{
+		switch (event->keyval) {
+		case GDK_Left:
+		case GDK_Right:
+			return totem_embedded_handle_key_press (emb, event);
+		}
+	}
+
+	if (event->state & GDK_CONTROL_MASK ||
+	    event->state & GDK_MOD1_MASK ||
+	    event->state & GDK_MOD3_MASK ||
+	    event->state & GDK_MOD4_MASK ||
+	    event->state & GDK_MOD5_MASK)
+		return FALSE;
+
+	return totem_embedded_handle_key_press (emb, event);
+}
+
 static gboolean
 totem_embedded_construct (TotemEmbedded *emb,
 			  GdkNativeWindow xid,
 			  int width,
 			  int height)
 {
-	GtkWidget *child, *container, *pp_button, *vbut;
+	GtkWidget *child, *container, *image;
 	BvwUseType type;
 	GError *err = NULL;
 	GConfClient *gc;
-	int volume;
+	double volume;
+
+	emb->xml = totem_interface_load ("mozilla-viewer.ui", TRUE,
+					 GTK_WINDOW (emb->window), emb);
+	g_assert (emb->xml);
+
 
 	if (xid != 0) {
 		g_assert (!emb->hidden);
@@ -1389,22 +1636,12 @@ totem_embedded_construct (TotemEmbedded *emb,
 		/* Can't do anything before it's realized */
 		gtk_widget_realize (emb->window);
 
-		emb->xml = totem_interface_load_with_root ("mozilla-viewer.glade",
-				"content_box", _("Plugin") /* FIXME! */, TRUE,
-				GTK_WINDOW (emb->window));
-		g_assert (emb->xml);
-
-		child = glade_xml_get_widget (emb->xml, "content_box");
+		child = GTK_WIDGET (gtk_builder_get_object (emb->xml, "content_box"));
 		gtk_container_add (GTK_CONTAINER (emb->window), child);
 	} else {
 		g_assert (emb->hidden);
 
-		emb->xml = totem_interface_load_with_root ("mozilla-viewer.glade",
-				"window", _("Plugin") /* FIXME! */, TRUE,
-				GTK_WINDOW (emb->window));
-		g_assert (emb->xml);
-
-		emb->window = glade_xml_get_widget (emb->xml, "window");
+		emb->window = GTK_WIDGET (gtk_builder_get_object (emb->xml, "window"));
 	}
 
 	if (emb->hidden || emb->audioonly != FALSE)
@@ -1425,12 +1662,42 @@ totem_embedded_construct (TotemEmbedded *emb,
 	{
 		/* FIXME! */
 		/* FIXME construct and show error message */
-		totem_embedded_error_and_exit (_("The Totem plugin could not startup."), err != NULL ? err->message : _("No reason."), emb);
+		totem_embedded_error_and_exit (_("The Totem plugin could not be started."), err != NULL ? err->message : _("No reason."), emb);
 
 		if (err != NULL)
 			g_error_free (err);
 	}
 
+	/* Fullscreen setup */
+	emb->fs_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	gtk_widget_realize (emb->fs_window);
+	gtk_window_set_title (GTK_WINDOW (emb->fs_window), _("Totem Movie Player"));
+
+	emb->fs = totem_fullscreen_new (GTK_WINDOW (emb->fs_window));
+	totem_fullscreen_set_video_widget (emb->fs, emb->bvw);
+	g_signal_connect (G_OBJECT (emb->fs->exit_button), "clicked",
+			  G_CALLBACK (totem_embedded_on_fullscreen_exit), emb);
+
+	emb->pp_fs_button = GTK_WIDGET (gtk_tool_button_new_from_stock 
+					(GTK_STOCK_MEDIA_PLAY));
+	g_signal_connect (G_OBJECT (emb->pp_fs_button), "clicked",
+			  G_CALLBACK (on_play_pause), emb);
+	gtk_container_add (GTK_CONTAINER (emb->fs->buttons_box), emb->pp_fs_button);
+
+	/* Connect the keys */
+	gtk_widget_add_events (emb->fs_window, GDK_KEY_PRESS_MASK |
+			       GDK_KEY_RELEASE_MASK);
+	g_signal_connect (G_OBJECT(emb->fs_window), "key_press_event",
+			G_CALLBACK (totem_embedded_key_press_event), emb);
+	g_signal_connect (G_OBJECT(emb->fs_window), "key_release_event",
+			G_CALLBACK (totem_embedded_key_press_event), emb);
+	gtk_widget_add_events (GTK_WIDGET (emb->bvw), GDK_KEY_PRESS_MASK | 
+			       GDK_KEY_RELEASE_MASK);
+	g_signal_connect (G_OBJECT(emb->bvw), "key_press_event",
+			G_CALLBACK (totem_embedded_key_press_event), emb);
+	g_signal_connect (G_OBJECT(emb->bvw), "key_release_event",
+			G_CALLBACK (totem_embedded_key_press_event), emb);
+
 	g_signal_connect (G_OBJECT(emb->bvw), "got-redirect",
 			G_CALLBACK (on_got_redirect), emb);
 	g_signal_connect (G_OBJECT (emb->bvw), "eos",
@@ -1441,8 +1708,13 @@ totem_embedded_construct (TotemEmbedded *emb,
 			G_CALLBACK (on_video_button_press_event), emb);
 	g_signal_connect (G_OBJECT(emb->bvw), "tick",
 			G_CALLBACK (on_tick), emb);
+	g_signal_connect (G_OBJECT (emb->bvw), "buffering",
+			  G_CALLBACK (on_buffering), emb);
 
-	container = glade_xml_get_widget (emb->xml, "video_box");
+	g_signal_connect (G_OBJECT (emb->bvw), "notify::volume",
+			  G_CALLBACK (property_notify_cb_volume), emb);
+
+	container = GTK_WIDGET (gtk_builder_get_object (emb->xml, "video_box"));
 	if (type == BVW_USE_TYPE_VIDEO) {
 		gtk_container_add (GTK_CONTAINER (container), GTK_WIDGET (emb->bvw));
 		/* FIXME: why can't this wait until the whole window is realised? */
@@ -1453,47 +1725,56 @@ totem_embedded_construct (TotemEmbedded *emb,
 		gtk_widget_hide (container);
 	}
 
-	emb->seek = glade_xml_get_widget (emb->xml, "time_hscale");
+	emb->seek = GTK_WIDGET (gtk_builder_get_object (emb->xml, "time_hscale"));
 	emb->seekadj = gtk_range_get_adjustment (GTK_RANGE (emb->seek));
+	gtk_range_set_adjustment (GTK_RANGE (emb->fs->seek), emb->seekadj);
 	g_signal_connect (emb->seek, "button-press-event",
 			  G_CALLBACK (on_seek_start), emb);
 	g_signal_connect (emb->seek, "button-release-event",
 			  G_CALLBACK (cb_on_seek), emb);
+	g_signal_connect (emb->fs->seek, "button-press-event",
+			  G_CALLBACK (on_seek_start), emb);
+	g_signal_connect (emb->fs->seek, "button-release-event",
+			  G_CALLBACK (cb_on_seek), emb);
 
-	pp_button = glade_xml_get_widget (emb->xml, "pp_button");
-	g_signal_connect (G_OBJECT (pp_button), "clicked",
+	emb->pp_button = GTK_WIDGET (gtk_builder_get_object (emb->xml, "pp_button"));
+	image = gtk_image_new_from_stock (GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_MENU);
+	gtk_button_set_image (GTK_BUTTON (emb->pp_button), image);
+	g_signal_connect (G_OBJECT (emb->pp_button), "clicked",
 			  G_CALLBACK (on_play_pause), emb);
 
 	gc = gconf_client_get_default ();
-	volume = gconf_client_get_int (gc, GCONF_PREFIX"/volume", NULL);
+	volume = ((double) gconf_client_get_int (gc, GCONF_PREFIX"/volume", NULL)) / 100.0;
 	g_object_unref (G_OBJECT (gc));
 
-	bacon_video_widget_set_volume (emb->bvw, volume);
-	vbut = glade_xml_get_widget (emb->xml, "volume_button");
-	bacon_volume_button_set_value (BACON_VOLUME_BUTTON (vbut), volume);
-	g_signal_connect (G_OBJECT (vbut), "value-changed",
+	emb->volume = GTK_WIDGET (gtk_builder_get_object (emb->xml, "volume_button"));
+	gtk_scale_button_set_adjustment (GTK_SCALE_BUTTON (emb->fs->volume),
+					 gtk_scale_button_get_adjustment 
+					 (GTK_SCALE_BUTTON (emb->volume)));
+	gtk_button_set_focus_on_click (GTK_BUTTON (emb->volume), FALSE);
+	gtk_scale_button_set_value (GTK_SCALE_BUTTON (emb->volume), volume);
+	g_signal_connect (G_OBJECT (emb->volume), "value-changed",
 			  G_CALLBACK (cb_vol), emb);
-	gtk_widget_add_events (vbut, GDK_SCROLL_MASK);
-	g_signal_connect (G_OBJECT (vbut), "scroll_event",
-			G_CALLBACK (on_volume_scroll_event), emb);
+	bacon_video_widget_set_volume (emb->bvw, volume);
 
-	emb->statusbar = TOTEM_STATUSBAR (glade_xml_get_widget (emb->xml, "statusbar"));
+	emb->statusbar = TOTEM_STATUSBAR (gtk_builder_get_object (emb->xml, "statusbar"));
+	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (emb->statusbar), FALSE);
 
-	gtk_widget_set_size_request (emb->window, width, height);
+	if (!emb->hidden)
+		gtk_widget_set_size_request (emb->window, width, height);
 
-#if GNOME_ENABLE_DEBUG
-	child = glade_xml_get_widget (emb->xml, "controls");
-	g_signal_connect_after (child, "size-allocate", G_CALLBACK (controls_size_allocate_cb), NULL);
+#ifdef GNOME_ENABLE_DEBUG
+	child = GTK_WIDGET (gtk_builder_get_object (emb->xml, "controls"));
+	g_signal_connect_after (G_OBJECT (child), "size-allocate", G_CALLBACK (controls_size_allocate_cb), NULL);
 #endif
 
 	if (emb->controller_hidden != FALSE) {
-		child = glade_xml_get_widget (emb->xml, "controls");
+		child = GTK_WIDGET (gtk_builder_get_object (emb->xml, "controls"));
 		gtk_widget_hide (child);
 	}
 
 	if (!emb->show_statusbar) {
-		child = glade_xml_get_widget (emb->xml, "statusbar");
-		gtk_widget_hide (child);
+		gtk_widget_hide (GTK_WIDGET (emb->statusbar));
 	}
 
 	/* Try to make controls smaller */
@@ -1503,14 +1784,12 @@ totem_embedded_construct (TotemEmbedded *emb,
 		rcstyle = gtk_rc_style_new ();
 		rcstyle->xthickness = rcstyle->ythickness = 0;
 
-		child = glade_xml_get_widget (emb->xml, "pp_button");
-		gtk_widget_modify_style (child, rcstyle);
+		gtk_widget_modify_style (emb->pp_button, rcstyle);
 		
-		child = glade_xml_get_widget (emb->xml, "time_hscale");
+		child = GTK_WIDGET (gtk_builder_get_object (emb->xml, "time_hscale"));
 		gtk_widget_modify_style (child, rcstyle);
 
-		child = glade_xml_get_widget (emb->xml, "volume_button");
-		gtk_widget_modify_style (child, rcstyle);
+		gtk_widget_modify_style (emb->volume, rcstyle);
 
 		g_object_unref (rcstyle);
 	}
@@ -1522,20 +1801,10 @@ totem_embedded_construct (TotemEmbedded *emb,
 	}
 
 	/* popup */
-	emb->menuxml = totem_interface_load_with_root ("mozilla-viewer.glade",
-			"menu", _("Menu"), TRUE,
-			GTK_WINDOW (emb->window));
+	emb->menuxml = totem_interface_load ("mozilla-viewer.ui", TRUE,
+					     GTK_WINDOW (emb->window), emb);
 	g_assert (emb->menuxml);
 
-	child = glade_xml_get_widget (emb->menuxml, "about1");
-	g_signal_connect (G_OBJECT (child), "activate",
-			  G_CALLBACK (on_about1_activate), emb);
-	child = glade_xml_get_widget (emb->menuxml, "copy_location1");
-	g_signal_connect (G_OBJECT (child), "activate",
-			G_CALLBACK (on_copy_location1_activate), emb);
-	child = glade_xml_get_widget (emb->menuxml, "preferences1");
-	gtk_widget_hide (child);
-
 	/* Create cursor and set the logo */
 	if (!emb->hidden) {
 		emb->cursor = gdk_cursor_new_for_display
@@ -1544,6 +1813,9 @@ totem_embedded_construct (TotemEmbedded *emb,
 		totem_embedded_set_logo_by_name (emb, "totem");
 	}
 
+	if (!emb->hidden && emb->autostart == FALSE)
+		totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
+
 	return TRUE;
 }
 
@@ -1634,10 +1906,21 @@ totem_embedded_unset_window (TotemEmbedded *embedded,
 	return TRUE;
 }
 
+static void
+totem_pl_item_free (gpointer data, gpointer user_data)
+{
+	TotemPlItem *item = (TotemPlItem *) data;
+
+	if (!item)
+		return;
+	g_free (item->uri);
+	g_free (item);
+}
+
 static void
 totem_embedded_clear_playlist (TotemEmbedded *embedded)
 {
-	g_list_foreach (embedded->playlist, (GFunc) g_free, NULL);
+	g_list_foreach (embedded->playlist, (GFunc) totem_pl_item_free, NULL);
 	g_list_free (embedded->playlist);
 
 	embedded->playlist = NULL;
@@ -1645,14 +1928,41 @@ totem_embedded_clear_playlist (TotemEmbedded *embedded)
 	embedded->num_items = 0;
 }
 
+/* FIXME this should live in the playlist parser */
+static int
+totem_embedded_parse_duration (const char *duration)
+{
+	int hours, minutes, seconds, tenths;
+
+	if (!duration)
+		return -1;
+
+	if (sscanf (duration, "%d:%d:%d", &hours, &minutes, &seconds) == 3)
+		return hours * 3600 + minutes * 60 + seconds;
+	if (sscanf (duration, "%d:%d.%d", &minutes, &seconds, &tenths) == 3)
+		return minutes * 60 + seconds;
+
+	return -1;
+}
+
 static void
-entry_added (TotemPlParser *parser,
+entry_metadata_foreach (const char *key,
+			const char *value,
+			gpointer data)
+{
+	if (g_ascii_strcasecmp (key, "url") == 0)
+		return;
+	g_print ("\t%s = '%s'\n", key, value);
+}
+
+static void
+entry_parsed (TotemPlParser *parser,
 	     const char *uri,
-	     const char *title,
-	     const char *genre,
+	     GHashTable *metadata,
 	     gpointer data)
 {
 	TotemEmbedded *emb = (TotemEmbedded *) data;
+	TotemPlItem *item;
 
 	if (g_str_has_prefix (uri, "file://") != FALSE
 	    && g_str_has_prefix (emb->base_uri, "file://") == FALSE) {
@@ -1660,12 +1970,15 @@ entry_added (TotemPlParser *parser,
 		return;
 	}
 
-	g_print ("added URI '%s' with title '%s' genre '%s'\n", uri,
-			title ? title : "empty", genre);
+	g_print ("added URI '%s'\n", uri);
+	g_hash_table_foreach (metadata, (GHFunc) entry_metadata_foreach, NULL);
 
-	//FIXMEchpe
-	//FIXME need new struct to hold that
-	emb->playlist = g_list_prepend (emb->playlist, g_strdup (uri));
+	item = g_new0 (TotemPlItem, 1);
+	item->uri = g_strdup (uri);
+	item->duration = totem_embedded_parse_duration (g_hash_table_lookup (metadata, "duration"));
+	item->starttime = totem_embedded_parse_duration (g_hash_table_lookup (metadata, "starttime"));
+
+	emb->playlist = g_list_prepend (emb->playlist, item);
 }
 
 static gboolean
@@ -1682,7 +1995,7 @@ totem_embedded_push_parser (gpointer data)
 	g_object_set (parser, "force", TRUE,
 		      "disable-unsafe", TRUE,
 		      NULL);
-	g_signal_connect (parser, "entry", G_CALLBACK (entry_added), emb);
+	g_signal_connect (parser, "entry-parsed", G_CALLBACK (entry_parsed), emb);
 	res = totem_pl_parser_parse_with_base (parser, emb->current_uri,
 					       emb->base_uri, FALSE);
 	g_object_unref (parser);
@@ -1728,31 +2041,6 @@ totem_embedded_push_parser (gpointer data)
 	return FALSE;
 }
 
-G_MODULE_EXPORT GtkWidget *
-totem_volume_create (void)
-{
-	GtkWidget *widget;
-
-	widget = bacon_volume_button_new (GTK_ICON_SIZE_MENU,
-					  0, 100, 1);
-	gtk_button_set_focus_on_click (GTK_BUTTON (widget), FALSE);
-	gtk_widget_show (widget);
-
-	return widget;
-}
-
-G_MODULE_EXPORT GtkWidget *
-totem_statusbar_create (void)
-{
-	GtkWidget *widget;
-
-	widget = totem_statusbar_new ();
-	totem_statusbar_set_has_resize_grip (TOTEM_STATUSBAR (widget), FALSE);
-	gtk_widget_show (widget);
-
-	return widget;
-}
-
 static char *arg_user_agent = NULL;
 static char *arg_mime_type = NULL;
 static char **arg_remaining = NULL;
