@@ -47,6 +47,7 @@
 #include "totem-pl-parser-xspf.h"
 #include "totem-pl-parser-media.h"
 #include "totem-pl-parser-smil.h"
+#include "totem-pl-parser-pla.h"
 #include "totem-pl-parser-lines.h"
 #include "totem-pl-parser-misc.h"
 #include "totem-pl-parser-private.h"
@@ -449,11 +450,18 @@ totem_pl_parser_line_is_empty (const char *line)
 gboolean
 totem_pl_parser_write_string (GnomeVFSHandle *handle, const char *buf, GError **error)
 {
-	GnomeVFSResult res;
-	GnomeVFSFileSize written;
 	guint len;
 
 	len = strlen (buf);
+	return totem_pl_parser_write_buffer (handle, buf, len, error);
+}
+
+gboolean
+totem_pl_parser_write_buffer (GnomeVFSHandle *handle, const char *buf, guint len, GError **error)
+{
+	GnomeVFSResult res;
+	GnomeVFSFileSize written;
+
 	res = gnome_vfs_write (handle, buf, len, &written);
 	if (res != GNOME_VFS_OK || written < len) {
 		g_set_error (error,
@@ -559,6 +567,9 @@ totem_pl_parser_write_with_title (TotemPlParser *parser, GtkTreeModel *model,
 	case TOTEM_PL_PARSER_XSPF:
 		return totem_pl_parser_write_xspf (parser, model, func,
 				output, title, user_data, error);
+	case TOTEM_PL_PARSER_IRIVER_PLA:
+		return totem_pl_parser_write_pla (parser, model, func,
+				output, title, user_data, error);
 	default:
 		g_assert_not_reached ();
 	}
@@ -672,6 +683,10 @@ totem_pl_parser_init (TotemPlParser *parser)
 				     "Title of the item to be added", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("author", "author",
+				     "Author of the item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("genre", "genre",
 				     "Genre of the item to be added", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
@@ -696,6 +711,22 @@ totem_pl_parser_init (TotemPlParser *parser)
 				     "String representing the start time of the stream (initial seek)", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("copyright", "copyright",
+				     "Copyright of the item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("abstract", "abstract",
+				     "Abstract of the item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("screensize", "screensize",
+				     "String representing the default movie size (double, full or original)", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("ui-mode", "ui-mode",
+				     "String representing the default UI mode (only compact is supported)", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (parser->priv->pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 }
 
 static void
@@ -764,10 +795,21 @@ totem_pl_parser_add_url_valist (TotemPlParser *parser,
 		/* Ignore empty values */
 		string = g_value_get_string (&value);
 		if (string != NULL && string[0] != '\0') {
+			char *fixed = NULL;
+
+			if (g_utf8_validate (string, -1, NULL) == FALSE) {
+				fixed = g_convert (string, -1, "UTF-8", "ISO8859-1", NULL, NULL, NULL);
+				if (fixed == NULL) {
+					g_value_unset (&value);
+					name = va_arg (var_args, char*);
+					continue;
+				}
+			}
+
 			/* Add other values to the metadata hashtable */
 			g_hash_table_insert (metadata,
 					     g_strdup (name),
-					     g_value_dup_string (&value));
+					     g_strdup (fixed ? fixed : string));
 		}
 
 		g_value_unset (&value);
@@ -810,19 +852,57 @@ totem_pl_parser_add_one_url (TotemPlParser *parser, const char *url, const char
 				 NULL);
 }
 
+static char *
+totem_pl_parser_remove_filename (const char *url)
+{
+	char *no_frag, *no_file, *no_qmark, *qmark;
+
+	no_frag = gnome_vfs_make_uri_canonical_strip_fragment (url);
+	qmark = strrchr (no_frag, '?');
+	if (qmark == NULL)
+		return no_frag;
+	no_qmark = g_strndup (no_frag, qmark - no_frag);
+	no_file = totem_pl_parser_base_url (no_qmark);
+	g_free (no_qmark);
+	g_free (no_frag);
+
+	return no_file;
+}
+
 char *
 totem_pl_resolve_url (const char *base, const char *url)
 {
 	GnomeVFSURI *base_uri, *new;
-	char *resolved;
+	char *resolved, *base_no_frag;
+
+	g_return_val_if_fail (url != NULL, NULL);
+	g_return_val_if_fail (base != NULL, g_strdup (url));
 
 	/* If the URI isn't relative, just leave */
 	if (strstr (url, "://") != NULL)
 		return g_strdup (url);
 
-	base_uri = gnome_vfs_uri_new (base);
+	/* Strip fragment and filename */
+	base_no_frag = totem_pl_parser_remove_filename (base);
+
+	/* gnome_vfs_uri_append_path is trying to be clever and
+	 * merges paths that look like they're the same */
+	if (url[0] != '/') {
+		char *newbase = g_strdup_printf ("%s/", base_no_frag);
+		base_uri = gnome_vfs_uri_new (newbase);
+		g_free (newbase);
+	} else {
+		base_uri = gnome_vfs_uri_new (base_no_frag);
+	}
+	g_free (base_no_frag);
+
 	g_return_val_if_fail (base_uri != NULL, g_strdup (url));
-	new = gnome_vfs_uri_resolve_relative (base_uri, url);
+
+	if (url[0] == '/')
+		new = gnome_vfs_uri_resolve_symbolic_link (base_uri, url);
+	else
+		new = gnome_vfs_uri_append_path (base_uri, url);
+
 	g_return_val_if_fail (new != NULL, g_strdup (url));
 	gnome_vfs_uri_unref (base_uri);
 	resolved = gnome_vfs_uri_to_string (new, GNOME_VFS_URI_HIDE_NONE);
@@ -856,6 +936,7 @@ static PlaylistTypes special_types[] = {
 	PLAYLIST_TYPE ("text/uri-list", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list, FALSE),
 	PLAYLIST_TYPE ("text/x-google-video-pointer", totem_pl_parser_add_gvp, NULL, FALSE),
 	PLAYLIST_TYPE ("text/google-video-pointer", totem_pl_parser_add_gvp, NULL, FALSE),
+	PLAYLIST_TYPE ("audio/x-iriver-pla", totem_pl_parser_add_pla, NULL, FALSE),
 #ifndef TOTEM_PL_PARSER_MINI
 	PLAYLIST_TYPE ("application/x-desktop", totem_pl_parser_add_desktop, NULL, TRUE),
 	PLAYLIST_TYPE ("application/x-gnome-app-info", totem_pl_parser_add_desktop, NULL, TRUE),
@@ -1193,6 +1274,8 @@ totem_pl_parser_can_parse_from_data (const char *data,
 		}
 	}
 
+	D(g_message ("Is unsupported mime-type '%s'", mimetype));
+
 	return FALSE;
 }
 
