@@ -494,7 +494,7 @@ on_recent_file_item_activated (GtkAction *action,
 	totem_signal_block_by_data (totem->playlist, totem);
 
 	end = totem_playlist_get_last (totem->playlist);
-	playlist_changed = totem_playlist_add_mrl (totem->playlist, uri, NULL);
+	playlist_changed = totem_playlist_add_mrl_with_cursor (totem->playlist, uri, NULL);
 	gtk_recent_manager_add_item (totem->recent_manager, uri);
 
 	totem_signal_unblock_by_data (totem->playlist, totem);
@@ -1278,7 +1278,7 @@ static const GtkToggleActionEntry toggle_entries[] = {
 	{ "shuffle-mode", NULL, N_("Shuff_le Mode"), NULL, N_("Set the shuffle mode"), G_CALLBACK (shuffle_mode_action_callback), FALSE },
 	{ "deinterlace", NULL, N_("_Deinterlace"), "I", N_("Deinterlace"), G_CALLBACK (deinterlace_action_callback), FALSE },
 	{ "show-controls", NULL, N_("Show _Controls"), "H", N_("Show controls"), G_CALLBACK (show_controls_action_callback), TRUE },
-	{ "sidebar", NULL, N_("_Sidebar"), "F9", N_("Show or hide the sidebar"), G_CALLBACK (show_sidebar_action_callback), TRUE }
+	{ "sidebar", NULL, N_("S_idebar"), "F9", N_("Show or hide the sidebar"), G_CALLBACK (show_sidebar_action_callback), TRUE }
 };
 
 static const GtkRadioActionEntry aspect_ratio_entries[] = {
